<?php

namespace plugin\cms\controller;

use plugin\cms\model\CmsCategory;
use plugin\cms\model\CmsTutorial;
use think\admin\Controller;
use think\admin\helper\QueryHelper;

/**
 * 办事指南管理
 */
class Tutorial extends Controller
{
    /**
     * 办事指南列表
     * @return void
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\DbException
     * @throws \think\db\exception\ModelNotFoundException
     */
    public function index()
    {
        $this->title = '办事指南';
        CmsTutorial::mQuery()->layTable(function () {
        }, static function (QueryHelper $query) {
            $query->equal('status')->like('title');
            $query->timeBetween('create_at');
        });
    }

    /**
     * 添加办事指南
     */
    public function add()
    {
        $this->title = '添加办事指南';
        $this->cates = CmsCategory::query()->where("pid", "=", 1)->select(); // 获取分类列表
        $this->cate_ids = [];
        CmsTutorial::mForm('form');
    }

    /**
     * 编辑办事指南
     */
    public function edit()
    {
        $this->title = '编辑办事指南';
        $this->id = $this->request->param('id');
        $this->cates = CmsCategory::query()->where("pid", "=", 1)->select(); // 获取分类列表
        $this->cate_ids = CmsTutorial::find($this->id)->category_ids; // 获取当前分类ID
        CmsTutorial::mForm('form', "id");
    }

    /**
     * 删除办事指南
     * @return void
     */
    public function remove()
    {
        CmsTutorial::mDelete();
    }

    /**
     * 修改办事指南状态
     * @return void
     */
    public function status()
    {
        CmsTutorial::mSave($this->_vali([
            'id.require' => '指南ID不能为空',
            'status.in:0,1' => '状态值范围异常！',
            'status.require' => '状态值不能为空！',
        ]), 'id');
    }

    /**
     * 修改办事指南排序
     * @return void
     */
    public function sort()
    {
        CmsTutorial::mSave($this->_vali([
            'id.require' => '指南ID不能为空',
            'sort.require' => '排序值不能为空！',
            'sort.number' => '排序必须为数字！',
            'sort.between:0,9999' => '排序值必须为0~9999之间！',
        ]), 'id');
    }

    /**
     * 表单数据过滤
     */
    protected function _form_result(bool $state, array $post)
    {
        if ($this->request->isPost()) {
            // 处理分类数据
            $tutorial = CmsTutorial::find($post['id']);
            if ($state) {
                $tutorial->category_ids = input('post.category_ids/a', []);;
            }
        }
    }
}