<?php

namespace plugin\cms\controller\api\auth;

use plugin\cms\controller\api\Auth;
use plugin\cms\model\CmsThumb;

class Thumb extends Auth
{
    /**
     * 获取点赞城市列表
     * @return void
     */
    public function index()
    {
        $thumbList = CmsThumb::mk()->scope('published')->order('create_at desc')->withCount([
            'like' => function ($query, &$alias) {
                $alias = 'my_like';
                $query->where('user_id', $this->usid);
            }
        ])->paginate();
        $this->success('获取点赞城市列表', $thumbList);
    }

    /**
     * 获取当前用户点赞的城市列表
     * @return void
     */
    public function my()
    {
        $myThumbs = CmsThumb::mk()->scope('published')->order('create_at desc')->where('user_id', '=', $this->usid)->paginate();
        $this->success('获取点赞城市成功', $myThumbs);
    }

    /**
     * 添加点赞城市
     * @return void
     */
    public function add()
    {
        $data = $this->_vali([
            'content.require' => '内容不能为空',
            'images.default' => ''
        ]);
        if ($data['images']) {
            $img_array = str2arr($data['images']);
            $data['thumb'] = $img_array[0];
        }
        $data['user_id'] = $this->usid;
        $data['status'] = 0;
        $thumb = CmsThumb::mk()->save($data);
        $this->success('点赞成功', $thumb);
    }

    /**
     * 获取点赞城市详情
     * @return void
     */
    public function info()
    {
        $id = $this->request->get('id');
        $thumb = CmsThumb::mk()->scope('published')->where('id', $id)->find();
        if (empty($thumb)) {
            $this->error('点赞城市不存在');
        }
        $thumb->inc("view_count");
        $thumb->save();
        $this->success('获取点赞城市详情', $thumb);
    }

    /**
     * 对点赞城市进行点赞操作
     * @return void
     */
    public function like()
    {
        $id = $this->request->post('id');
        $thumb = CmsThumb::mk()->scope('published')->where('id', $id)->find();
        if (empty($thumb)) {
            $this->error('点赞城市不存在');
        }
        $like = $thumb->like()->save([
            'user_id' => $this->usid
        ]);
        $thumb->inc("like_count");
        $thumb->save();
        $this->success('点赞成功');
    }
}