<?php

namespace plugin\points_mall\controller;

use plugin\account\model\PluginAccountBind;
use plugin\points_mall\model\PointsMallOrder;
use plugin\points_mall\model\PointsMallOrderSender;
use think\admin\Controller;
use think\admin\helper\QueryHelper;

/**
 * 订单发货管理
 * @class Sender
 * @package plugin\wemall\controller\shop
 */
class Sender extends Controller
{
    /**
     * 订单状态
     * @var int[]
     */
    private $oStatus = [4, 5, 6, 7];

    /**
     * 订单发货管理
     * @auth true
     * @menu true
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\DbException
     * @throws \think\db\exception\ModelNotFoundException
     */
    public function index()
    {
        $this->type = trim(input('type', 'ta'), 't');
        PointsMallOrderSender::mQuery()->layTable(function () {
            $this->title = '订单发货管理';
            $this->total = ['t0' => 0, 't1' => 0, 't2' => 0, 'ta' => 0];
            $this->address = sysdata('plugin.wemall.address');
            // 订单状态统计
            $order = PointsMallOrder::mk()->whereIn('status', $this->oStatus)->where(['delivery_type' => 1]);
            $query = PointsMallOrderSender::mk()->whereRaw("order_no in {$order->field('order_no')->buildSql()}");
            foreach ($query->fieldRaw('status,count(1) total')->group('status')->cursor() as $vo) {
                $this->total["ta"] += $vo['total'];
                $this->total["t{$vo['status']}"] = $vo['total'];
            }
        }, function (QueryHelper $query) {
            $query->with(['user', 'main']);
            $query->like('user_name|user_phone#user_name,region_prov|region_city|region_area|region_addr#address');
            $query->dateBetween('create_time,express_time')->equal('status')->like('express_code,order_no');

            // 用户搜索查询
            $db = PluginAccountBind::mQuery()->like('phone|nickname#user_keys')->db();
            if ($db->getOptions('where')) $query->whereRaw("unid in {$db->field('id')->buildSql()}");

            // 订单搜索查询
            $db = PointsMallOrder::mk()->whereIn('status', $this->oStatus)->where(['delivery_type' => 1]);
            $query->whereRaw("order_no in {$db->field('order_no')->buildSql()}");

            // 列表选项卡状态
            if (is_numeric($this->type)) {
                $query->where(['status' => $this->type]);
            }
        });
    }

    /**
     * 快递发货地址
     * @auth true
     * @throws \think\admin\Exception
     */
    public function config()
    {
        if ($this->request->isGet()) {
            $this->vo = sysdata('plugin.wemall.address');
            $this->fetch();
        } else {
            sysdata('plugin.wemall.address', $this->request->post());
            $this->success('地址保存成功！');
        }
    }

    /**
     * 修改快递管理
     * @auth true
     */
    public function delivery()
    {
        PointsMallOrderSender::mForm('delivery_form', 'order_no');
    }

    /**
     * 快递表单处理
     * @param array $vo
     */
    protected function _delivery_form_filter(array &$vo)
    {
        if ($this->request->isGet()) {
            $map = ['code' => $vo['delivery_code'], 'status' => 1, 'deleted' => 0];
        } elseif ($this->request->isPost()) {
            $map = ['order_no' => $vo['order_no']];
            $order = PointsMallOrder::mk()->where($map)->findOrEmpty();
            if ($order->isEmpty()) $this->error('订单查询异常，请稍候再试！');

            // 追加表单数据
            $vo['status'] = 2;
//            $vo['company_name'] = "";
            $vo['express_time'] = $vo['express_time'] ?? date('Y-m-d H:i:s');

            $vo['region_prov'] = $vo['form_prov'] ?? '';
            $vo['region_city'] = $vo['form_city'] ?? '';
            $vo['region_area'] = $vo['form_area'] ?? '';

            // 更新订单发货状态
            if ($order['status'] === 4) $order->save(['status' => 5]);
        }
    }
}