<?php

namespace plugin\points_mall\controller\api;

use plugin\points_mall\model\PointsMallGoods;
use plugin\points_mall\model\PointsMallGoodsCate;
use plugin\points_mall\model\PointsMallGoodsItem;
use think\admin\Controller;
use think\admin\extend\CodeExtend;
use think\admin\helper\QueryHelper;
use think\db\Query;

/**
 * 商品数据管理
 * @class Goods
 * @package plugin\points_mall\controller\api
 */
class Goods extends Controller
{
    /**
     * 获取商品列表或详情
     * @return void
     */
    public function get()
    {
        $this->cnames = null;
        PointsMallGoods::mQuery(null, function (QueryHelper $query) {
            // 显示分类显示
            if (!empty($vCates = input('cates'))) {
                $cates = array_filter(PointsMallGoodsCate::items(), function ($v) use ($vCates) {
                    return $v['id'] == $vCates;
                });
                $this->cnames = null;
                if (count($cates) > 0) {
                    $cate = array_pop($cates);
                    $this->cnames = array_combine($cate['ids'], $cate['names']);
                }
            }
            $query->equal('code')->like('name#keys')->like('marks,cates', ',');
            if (!empty($code = input('code'))) {
                // 查询单个商品详情
                $query->with(['items']);
                PointsMallGoods::mk()->where(['code' => $code])->inc('num_read')->update([]);
            }
            // 数据排序处理
            $sort = intval(input('sort', 0));
            $type = intval(input('order', 0)) ? 'asc' : 'desc';
            if ($sort === 1) {
                $query->order("num_read {$type},sort {$type},id {$type}");
            } elseif ($sort === 2) {
                $query->order("price_selling {$type},sort {$type},id {$type}");
            } else {
                $query->order("sort {$type},id {$type}");
            }
            $query->where(['status' => 1, 'deleted' => 0]);
            // 查询数据分页
            $page = intval(input('page', 1));
            $limit = max(min(intval(input('limit', 20)), 60), 1);
            $this->success('获取商品数据', $query->page($page, false, false, $limit));
        });
    }

    /**
     * 数据结果处理
     * @param array $data
     * @param array $result
     * @return void
     */
    protected function _get_page_filter(array &$data, array &$result)
    {
        $result['cnames'] = $this->cnames ?? null;
    }

    /**
     * 获取商品分类及标签
     * @return void
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\DbException
     * @throws \think\db\exception\ModelNotFoundException
     */
    public function cate()
    {
        $this->success('获取分类成功', [
            'cate' => PointsMallGoodsCate::dtree(),
        ]);
    }


}