<?php

namespace plugin\points_mall\model;

use think\admin\Model;

class PointsMallGoods extends Model
{

    /**
     * 日志名称
     * @var string
     */
    protected $oplogName = '商品';

    /**
     * 日志类型
     * @var string
     */
    protected $oplogType = '分销商城管理';

    /**
     * 关联产品规格
     * @return \think\model\relation\HasMany
     */
    public function items()
    {
        return static::mk()
            ->hasMany(PointsMallGoodsItem::class, 'gcode', 'code')
            ->withoutField('id,status,create_time,update_time')
            ->where(['status' => 1]);
    }

    /**
     * 关联产品列表
     * @return array
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\DbException
     * @throws \think\db\exception\ModelNotFoundException
     */
    public static function lists(): array
    {
        $model = static::mk()->with('items')->withoutField('specs');
        return $model->order('sort desc,id desc')->where(['deleted' => 0])->select()->toArray();
    }

    /**
     * 处理商品分类数据
     * @param mixed $value
     * @return array
     */
    public function getCatesAttr($value): array
    {
        $ckey = 'PointsMallGoodsCateItem';
        $cates = sysvar($ckey) ?: sysvar($ckey, PointsMallGoodsCate::items(true));
        $cateids = is_string($value) ? str2arr($value) : (array)$value;
        foreach ($cates as $cate) if (in_array($cate['id'], $cateids)) return $cate;
        return [];
    }

    /**
     * 设置轮播图片
     * @param mixed $value
     * @return string
     */
    public function setSliderAttr($value): string
    {
        return is_string($value) ? $value : (is_array($value) ? arr2str($value) : '');
    }

    /**
     * 获取轮播图片
     * @param mixed $value
     * @return array
     */
    public function getSliderAttr($value): array
    {
        return is_string($value) ? str2arr($value, '|') : [];
    }

}