<?php

namespace plugin\ticket\controller;

use plugin\ticket\model\TicketCommonReply;
use think\admin\Controller;
use think\admin\helper\QueryHelper;

/**
 * 常用回复管理
 */
class CommonReply extends Controller
{
    /**
     * 常用回复列表
     * @auth true
     * @menu true
     * @return void
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\DbException
     * @throws \think\db\exception\ModelNotFoundException
     */
    public function index()
    {
        $this->title = '常用回复列表';
        TicketCommonReply::mQuery()->layTable(function () {
        }, function (QueryHelper $query) {
            $query->like(['title#keyword'])
                ->dateBetween(['create_at'])
                ->equal(['status']);
            $query->append(['status_text']);
        });
    }

    /**
     * 添加常用回复
     * @auth true
     * @menu true
     * @return void
     */
    public function add()
    {
        $this->title = '添加常用回复';
        TicketCommonReply::mForm('form');
    }

    /**
     * 编辑常用回复
     * @auth true
     * @menu true
     * @return void
     */
    public function edit()
    {
        $this->title = '编辑常用回复';
        TicketCommonReply::mForm('form');
    }

    /**
     * 删除常用回复
     * @auth true
     * @menu true
     * @return void
     */
    public function remove()
    {
        TicketCommonReply::mDelete();
    }

    /**
     * 启用常用回复
     * @auth true
     * @menu true
     * @return void
     */
    public function enable()
    {
        $this->_applyFormToken();
        ['id' => $id] = $this->_vali(['id.require' => '请指定常用回复ID！']);
        TicketCommonReply::mk()->where(['id' => $id])->update(['status' => 1]);
        $this->success('启用成功！');
    }

    /**
     * 禁用常用回复
     * @auth true
     * @menu true
     * @return void
     */
    public function disable()
    {
        $this->_applyFormToken();
        ['id' => $id] = $this->_vali(['id.require' => '请指定常用回复ID！']);
        TicketCommonReply::mk()->where(['id' => $id])->update(['status' => 0]);
        $this->success('禁用成功！');
    }
}