<?php

namespace plugin\ticket\model;

use plugin\inspection\model\InspectionStaff;
use think\admin\Model;
use think\admin\model\SystemUser;

class TicketTicket extends Model
{
    protected $append = ['status_text', 'type_name', 'dept_name', 'imgs_arr'];
    protected $table = 'ticket_ticket';
    public function type()
    {
        return $this->belongsTo(TicketType::class, 'type_id');
    }

    public function approval()
    {
        return $this->belongsTo(ApprovalInstance::class, 'id', 'oid');
    }

    public function user()
    {
        return $this->morphTo('user');
    }

    public function reply()
    {
        return $this->hasMany(TicketReply::class, 'ticket_id')->order("create_at", 'asc');
    }

    public function getImgsArrAttr($value, $data)
    {
        if (!empty($data['imgs'])) {
            return str2arr($data['imgs'] ?: '', '|');
        } else {
            return [];
        }
    }

    public function getLastReplyAttr($value, $data)
    {
        return $this->reply()->order('create_at', 'desc')->find();
    }

    public function getSourceTypeNameAttr($value, $data)
    {
        if (!empty($this->getSourceTypeList()[$data['source_type']])) {
            return $this->getSourceTypeList()[$data['source_type']];
        } else {
            return '未知';
        }
    }

    public function getSourceTypeList()
    {
        return [
            0 => '管理人员创建',
            1 => '用户随手拍',
            2 => '维修人员上报',
        ];
    }

    public function getTypeNameAttr($value, $data)
    {
        $type = $this->type()->find();
        if (!empty($type)) {
            return $type['name'];
        } else {
            return '未知';
        }
    }


    public function getDeptNameAttr($value, $data)
    {
        $dept = $this->dept()->find();
        if (!empty($dept)) {
            return $dept['name'];
        } else {
            return '未知';
        }
    }

    public function scopeAvail($query)
    {
        return $query->where('status', '>=', '0');
    }

    public function userShares()
    {
        return $this->hasMany(TicketUserShare::class, 'linked_ticket_id');
    }

    public function dept()
    {
        return $this->belongsTo(TicketDept::class, 'dept_id');
    }

    public function fzUser()
    {
        return $this->belongsTo(SystemUser::class, 'fz_user_id');
    }

    public function verifys()
    {
        return $this->hasMany(TicketVerify::class, 'ticket_id')->order('create_at', 'desc');
    }

    public function repairs()
    {
        return $this->hasMany(TicketRepair::class, 'ticket_id')->order('create_at', 'desc');
    }

    public function views() {
        return $this->hasMany(TicketView::class, 'ticket_id')->order('create_at', 'desc');
    }

    public function viewProcess() {
        return $this->belongsTo(ApprovalInstance::class, 'view_pid', 'id');
    }

    public function repairProcess()
    {
        return $this->belongsTo(ApprovalInstance::class, 'repair_pid', 'id');
    }

    public function verifyProcess()
    {
        return $this->belongsTo(ApprovalInstance::class, 'verify_pid', 'id');
    }

    public function repair_accept()
    {
        return $this->hasMany(TicketRepairAccept::class, 'ticket_id')->order('create_at', 'desc');
    }

    public function scopeUnConf($query)
    {
        return $query->where('state', '=', '0');
    }

    public function scopeInter($query)
    {
        return $query->where('state', '=', '1');
    }

    public function scopeOuter($query)
    {
        return $query->where('state', '=', '2');
    }
}